	INCLUDE "p:\Cosc362\project\SoundRender.asm"
	INCLUDE "p:\Cosc362\project\Samples.asm"
	
****************** Variables *****************

myData: SECTION
	TrackPtr:	dc.w 0		; //
	TrackLength:	dc.w 0		; //
	TrackEndPtr:	dc.w 0		; //
	EventCode:	dc.b 0		; // current midi code
	SubEventCode:	dc.b 0		; // current midi subevent code
	Event:		dc.b 0		; // current midi event
	LastEventCode:	dc.b 0		; // last midi event
	EndPos:		dc.w 0		; // byte position of end of MTrk
	Division:	dc.w 0		; // deltas per quater note
	BeatsPerMinute:	dc.w 120	; // beats per minute
	samplesPerDelta:dc.w 0		; // unit of time for delta timing
	midiTime:	dc.w 0		; // 
	tempVlc:	dc.w 0		; // 
	tmpMidiPtr:	dc.w 0		; // 
	lastEventMidiPtr:dc.w 0		; // used in synchronizing midi events to buffer
	_temp1: 	dc.l 0		; // storage space for any function
	_temp2: 	dc.l 0		; //  that needs it
	
********************** CONSTANTS ******************************

text: SECTION

trackname:
	dc.b "playing track: mrsrobinson.mid"
	dc.b EOT

resetMidi:	; //******************************************************
	pshd
	pshx
	
	movw	#120, BeatsPerMinute ; // default as per midi standard
	movw	#0, bufferTimePos
	movw	#0, midiTime
	movw	#$FFFF, lastEventMidiPtr
	
	ldx	#channel0
	ldaa	#channel_count
	ldab	#channel_data_size
	
resetMidiLoop:
	clr x ; // chan\1_state:        dc.b 0 ; // 0 available, else envelope position
	abx
	dbne a, resetMidiLoop
	
	pulx
	puld
	rts

readMidiHeader:	; //******************************************************

	ldx	TrackPtr
	
	ldd	2,x+	; // check chunkName is correct, looking for "MThd"
	cpd	#$4D54	; // 'MT'
	lbne	readMidiHeaderBadChunkName
	ldd	2,x+
	cpd	#$6864	; 'hd'
	lbne	readMidiHeaderBadChunkName
	
			; // get chunkSize 
	ldd	2,x+	; // first 2 bytes must be 0 as we only have 64k of memory
	lbne	readMidiHeaderBadChunkLength
	ldd	2,x+
	cpd	#6	; // Header chunks are _always_ 6 long
	lbne	readMidiHeaderBadChunkLengthNot6
	
	ldd	2,x+	; // format 0 - single track file format.  ONLY
	lbne	readMidiHeaderUnsupportedFormat
	
	ldd	2,x+	; // ntrks should be 1 for format 0
	cpd	#1
	lbne	readMidiHeaderIncorrectTrackCount
	
	ldd	2,x+ 	; // Division
	lblt	divisionWrongFormat
	std	Division ; // deltas per quater note
			 ; // bpm*division*4 = dpm (deltas per minute)
			 ; // 60*11,000 / dpm = samples per delta
			 ; // 60*11,000/4 / bpm/division = samples per delta
			 ; // $28488 / (bpm*division)
	mul16	BeatsPerMinute, Division, _temp2
	
	movw	#rateh, _temp1  ; -> 12 khz
	movw	#ratel, _temp1+2 
	movw	#0, _temp2
	div32	_temp1, BeatsPerMinute, _temp2+2
	div32	_temp2, Division, samplesPerDelta
	ldd	samplesPerDelta
	
	stx	TrackPtr
	
	rts

readMidiTrack:	; // ******************************************************
	ldx	TrackPtr
					; // check chunkName is correct
	ldd	2,x+			; // looking for "MTrk"
	cpd	#$4D54			; // 'MT'
	lbne	readMidiTrackBadChunkName
	ldd	2,x+
	cpd	#$726B			; // 'rk'
	lbne	readMidiTrackBadChunkName
	
					; // get track length
	ldd	2,x+			; // only 16bit processor
	lbne	trackLengthExceeds16Bits
	
	ldd	2,x+
	std	TrackLength
	
	tfr	x,d
	addd	TrackLength
	std	TrackEndPtr
	
	bclr    PORTDLC,#$F0
	
readMidiTrackLoop:
	pshx
	jsr	KEYSCAN
	pulx
	cmpa	#0
	beq	readMidiTrackLoop_notstop
	
	rts
	
readMidiTrackLoop_notstop:
	cpx	TrackEndPtr		; // while not end of track decode track
	lbge	endOfTrack		; // keep filling buffer
	
	; // if(bufferTimePos>0) fillbuffer; <- fill buffer
	ldd	bufferTimePos
	beq	notFillBuffer
	jsr	FILL_BUFFER
	bra	readMidiTrackLoop
notFillBuffer:
	; // else
	; // {
	; //   store midiPtr
	stx	tmpMidiPtr
	; //   getVlc
	jsr	getVlc			; // int delta = getVLC(d); // Get delta for next
	std	tempVlc
	; //   if(vlc==0 || lastEventMidiPtr==x) decodeMidi; <- we are currently updating channels
	; //   else
	lbeq	decodeMidi
	cpx	lastEventMidiPtr
	lbeq	decodeMidi
	
	addd	midiTime
	std	midiTime
	
	ldd	tempVlc
	ldy	samplesPerDelta
	emul	; // d holds next timepos
	
	cpy	#0
	beq	skipBufferTimePosError
	
	; // todo implement bufferTime sizes larger than a word
	
skipBufferTimePosError:
	; //   if(vlc!=0) bufferTimePos+=vlc; load midiPtr; <- we are currently updating next event ptr
	std	_temp1
	add32	bufferTimePos, _temp1, bufferTimePos; // timePos += delta; // update current track time
	; // }
	stx	lastEventMidiPtr
	ldx	tmpMidiPtr
	lbra	readMidiTrackLoop
	
decodeMidi:
	ldaa	1,x+			; // int eventCode = d.readByte()&0xFF;
	staa	EventCode

	; // refer midi docs for running status byte explanation
	bmi	readMidiTrackLoop_eventRunningStatusBit ; // if(eventCode<0x80) 
	;{
	;  // probably a running status byte
	;  // replace event with old event and go back 1
	dex
	ldaa	LastEventCode		; // eventCode = lastEventCode;
	staa	EventCode 		; // a now holds EventCode
	;}
	; // else
	bra	readMidiTrackLoop_skip1
readMidiTrackLoop_eventRunningStatusBit:
	; // { // a already holds EventCode
	staa	LastEventCode 		; // lastEventCode = eventCode;
	; // }
readMidiTrackLoop_skip1:
					; // Note: A currently holds eventCode ;  // Bn  cc  nn
	anda	#$F0			; // int ef0 = event&0xF0;
	staa	Event
	
	cmpa	#$80			; // if(ef0==0x80) // Note off 9n  kk  vv
	lbne	readMidiTrackLoop_else1
	
noteOnIsOff:
	ldaa	EventCode		; // ; { int midiChannel = event&0xF;
	anda	#$0F
	
	ldab	1,x+			; // int note = d.readByte()&0xFF;
	
	jsr	findChannelNoteOn	; // y holds channel now
	
	ldab	1,x+			; // int velocity = d.readByte()&0xFF;
	
	cpy	#0			; // if y is 0 then no channel was found with note on
	beq	noteAlreadyOff
	
	ldaa	#2			; // 2 means decay now
	staa	y
	ldd	#noteOffdecayRate
	std	6,y			; // set decay rate
	
noteAlreadyOff:
	lbra	readMidiTrackLoop_elseEnd ; // }

readMidiTrackLoop_else1: 
	cmpa	#$90 			; // else if(ef0==0x90) // Note on 9n  kk  vv
	lbne	readMidiTrackLoop_else2
	; // {
	
	jsr	findFreeChannel		; // free channel now in y
	ldaa	#1
	staa	0,y			; // set channel as in use
	
	ldab	EventCode		; // int midiChannel = event&0xF;
	andb	#$0F
	stab	1,y			; // store midi channel

	ldaa	1,x+			; // int note = d.readByte()&0xFF;
	staa	2,y			; // store note
	
	ldab	#$60			; // we do a rough adjustment for 
	mul				; // psychocoustic principles
	std	_temp3
	ldd	#$75FF ;
	subd	_temp3
	std	4,y	; envVolume

	pshx

	ldaa	2,y			; // reload note
	ldab	1,y			; // reload midi channel
	
	jsr 	findClosestSampleToNote ; // a now holds note
	; // x now holds note ptr, d holds sampleSubPosAdd
	pshd
	
	
	ldd	#$7000 	;
	staa	3,y	; volume
	; // vol
	ldd	#noteOndecayRate ; natural decay for instrument
	std	6,y	; envDelta
	ldd	1,x	; sample start
	std	8,y	; samplePtr
	ldd	3,x	; sample loop start
	std	10,y	; sample loop start
	ldd	5,x 	; sample end
	std	12,y	; sample loop end
	subd	3,x
	std	18,y	; sample loop diff
	clra
	clrb
	std	14,y	; sampleSubPos
	puld
	std	16,y	; sampleSubPosAdd
	
	pulx
	; // ignoring this atm
	ldab	1,x+			; // int velocity = d.readByte()&0xFF;
	
	bne	readMidiTrackLoop_VelocityNot0
	; // if(velocity==0) // using note on with vel=0 for a note off
	; // {               
	; // velocity = 0x40;
	ldab	#$40
	ldaa	#0
	staa	0,y
	dex
	dex
	lbra noteOnIsOff
	; //  }
readMidiTrackLoop_VelocityNot0:
	; velocity in b
	bra	readMidiTrackLoop_elseEnd
	; }

readMidiTrackLoop_else2:
	cmpa	#$D0			; // else if(ef0==0xD0) // Channel Key Pressure  Dn  ww
	bne	readMidiTrackLoop_else3	; // {
	; // Bn  cc  nn
	ldaa	EventCode		; // int midiChannel = event&0xF;
	anda	#$0F
	ldaa	1,x+			; // int pressure = d.readByte()&0xFF;
	;}
	bra readMidiTrackLoop_elseEnd
	
readMidiTrackLoop_else3:
	cmpa	#$B0			; // else if(ef0==0xB0) // MIDI Control Change Messages Bn cc nn
	bne	readMidiTrackLoop_else4	; // {
	
	; // Bn  cc  nn
	ldaa	EventCode		; // int midiChannel = event&0xF;
	anda	#$0F
	ldaa	1,x+			; // int controllerNumber = d.readByte()&0xFF;
	ldaa	1,x+			; // int controllerValue = d.readByte()&0xFF;
	; // }
	bra	readMidiTrackLoop_elseEnd
readMidiTrackLoop_else4:
	cmpa	#$C0			; // else if(ef0==0xC0) // Program Change Cn pp
	bne	readMidiTrackLoop_else5 ; // {
	; // Bn  cc  nn
	ldaa	EventCode 		; // int midiChannel = event&0xF;
	anda	#$0F
	
	ldaa	1,x+ 			; // int newProgramNumber = d.readByte()&0xFF;
	bra	readMidiTrackLoop_elseEnd ; // }
	
readMidiTrackLoop_else5:
	cmpa	#$E0			; // else if(ef0==0xE0) // Pitch Bend  En  lsb  msb
	bne	readMidiTrackLoop_else6 ; // {
	; // Bn  cc  nn
	ldaa	EventCode		; // int midiChannel = event&0xF;
	anda	#$0F
	ldd	2,x+			; // d.skip(2);
	bra	readMidiTrackLoop_elseEnd	; // }
	
readMidiTrackLoop_else6:
	ldaa	EventCode		; // else if(event==0xF0||event==0xF7)
	cmpa	#$F0	
	beq	readMidiTrackLoop_F0_or_F7
	cmpa	#$F7
	beq	readMidiTrackLoop_F0_or_F7
	bra	readMidiTrackLoop_else7
readMidiTrackLoop_F0_or_F7: 		; // {
	jsr getVlc			; // int length = getVLC(d);
	adx				; // d.skip(length);
	bra	readMidiTrackLoop_elseEnd ; // }
	
readMidiTrackLoop_else7:
	cmpa	#$FF
	lbne	readMidiTrackLoop_else_error
	
	ldaa	1,x+
	staa	SubEventCode

	cmpa	#$2F			; // end of track
	bne	readMidiTrackLoop_else7_skip
	
	jsr	resetMidi		; // end of track means all notes off

readMidiTrackLoop_else7_skip:
	jsr	getVlc
	adx 				; // d.skip(length);
					; // TODO: decode sub-event decode subevent
	bra readMidiTrackLoop_elseEnd

readMidiTrackLoop_elseEnd:
	lbra	readMidiTrackLoop
	rts

endOfTrack:
	stx	TrackPtr
	
	ldd	bufferTimePos
	beq	almostEnd
	jsr	FILL_BUFFER	; // wait for buffer to finish filling
	bra 	endOfTrack
almostEnd:
	jsr	killAllChannels

finalEnd:

	rts
	dc.b	"endOfTrack"		; // for debug
	
**************************** getVlc ************************************
	
getVlc: 			; // x holds current midi track location, result in d
	pshy
	clra
	ldab	1,x+		; // value = d.readByte()&0xFF;
	bpl getVlc_skip 	; // if( (value&0x80)>0) // top bit set
	; {
	andb #$7F 		; // value &= 0x7f;
	
getVlc_loop: ;	do
				; // {
				; // value = (value<<7)+ (c&0x7f);
				; // annoying this shift thing
	lsld
	bvs	getVlc_VlcTooBig
	lsld
	bvs	getVlc_VlcTooBig
	lsld
	bvs	getVlc_VlcTooBig
	lsld
	bvs	getVlc_VlcTooBig
	lsld
	bvs	getVlc_VlcTooBig
	lsld
	bvs	getVlc_VlcTooBig
	lsld
	bvs	getVlc_VlcTooBig
	tfr	d,y

	ldab	1,x+ 		; // c = d.readByte()&0xFF;
	tfr	b,a
	andb	#$7F
	aby
	
	tsta			; // } while( (c&0x80)>0);
	tfr	y,d
	bmi	getVlc_loop	; //  }
     
getVlc_skip:
	puly
	rts

************************ ERRORS ********************************

	errorMacro getVlc_VlcTooBig,'getVlc_VlcTooBig'
	errorMacro readMidiHeaderBadChunkName,'readMidiHeaderBadChunkName'
	errorMacro readMidiHeaderBadChunkLength,'readMidiHeaderBadChunkLength'
	errorMacro readMidiHeaderBadChunkLengthNot6,'readMidiHeaderBadChunkLengthNot6'
	errorMacro readMidiHeaderUnsupportedFormat,'readMidiHeaderUnsupportedFormat'
	errorMacro readMidiHeaderIncorrectTrackCount,'readMidiHeaderIncorrectTrackCount'
	errorMacro trackLengthExceeds16Bits,'trackLengthExceeds16Bits'
	errorMacro readMidiTrackLoop_else_error,'readMidiTrackLoop_else_error'
	errorMacro readMidiTrackBadChunkName,'readMidiTrackBadChunkName'
	errorMacro divisionWrongFormat,'DivisionWrongFormat'
	errorMacro bufferTimePosExceedsWord,'bufferTimePosExceedsWord'
