	XDEF INITLCD, LCDOUT, LCDCMD, LCDSTR, NUM_TO_LCD
	
;  LCD.ASM
;       LCD Display routines for Axiom HC12B development boards.
;
;  INITLCD:	Initialize the LCD display
;  LCDOUT:	Send a byte in A to the LCD data port
;  LCDCMD:	Send a byte in A to the LCD command port and wait
;  NUM_TO_LCD:  Converts num to ASCII and sends to LCD
;
; CHANGES: LCDSTR routine added by S. Weddell 17/5/01

; Initialize LCD display
myCode:SECTION
INITLCD:
	ldaa	#$3C 	; 2x40 Display
	jsr	LCDCMD	; write command
	ldaa	#$0f    ; Display and cursor on
	jsr	LCDCMD	; write command
	ldaa	#$14    ; shift cursor right
	jsr	LCDCMD	; write command
	ldaa	#$01    ; clear display and home cursor
	jsr	LCDCMD	; write command
	rts

CLEARLCD:
	ldaa	#$01    ; clear display and home cursor
	jsr	LCDCMD	; write command
	rts

LCDOUT:
        staa    $271    ; output data to lcd
	jsr	delay100us ; WAIT this is too long, use a shorter one
	rts

LCDSTR:
	ldaa	1,X+	; while (delimiter != '0') {
			;  if (delimiter == '0')
	beq	return	;   return
	jsr	LCDOUT	;  else
			;   output char
	bra	LCDSTR	; }
return:	rts

; Send a command in A to the LCD command register
LCDCMD:
        staa    $270    ; write command
	jsr	delay40 ; wait
	rts


; wait 100 microseconds (approx.)
delay100us:
	psha
	clra
loop100:
	deca
	bne	loop100
	pula
	rts

; wait appropriate time for LCD command process
delay40:
	pshx
	ldx	#$0000
loop40:
	dex
	bne	loop40
	pulx
	rts

; This routine will convert a hexadecimal number to ASCII and will 
; send this to the LCD for display. The number must be between 0 and 15 (0xf)
NUM_TO_LCD:
		psha		; save accumulator A
		tab		; copy A into B
		LSRA
		LSRA
		LSRA
		LSRA		; allign most significant number (MSN)
		anda	#$0f
		andb	#$0f	; mask off unwanted numbers
		cmpa	#$0a	; if (MSN >= 9)
		bhs	ALPHA	;  convert to alpha
		adda	#$30	; else convert to number
		bra	NEXT
ALPHA:		adda	#$57
NEXT:		cmpb	#$0a	; if (LSN >= 9)
		bhs	ALPHA2	;  convert to alpha
		addb	#$30	; else convert to number
		bra	NEXT2
ALPHA2:		addb	#$57
NEXT2:		pshb
		jsr	LCDOUT		; display adc value on LCD
		pulb
		tba		; get the least significant number (LSN)
		jsr	LCDOUT		; display adc value on LCD	
		pula		; restore accumulator A	
		RTS

