;***************************************************************************
;  SCI_MOD.ASM
; PURPOSE: This module should be used 
;	Simple program for testing the Axiom AX-BDM12 debugger using a
;	development board with external memory.
;
;	A simple text string is sent to the terminal using COM1.
;
;

;**************
;     RAM     *
;**************

myData: section
INBUFF:  ds.b  IBUFSIZ ;    input buffer, defined but not used
ENDBUFF: EQU  *
COUNT:   ds.b  1   ;  # characters read, also unused


     XDEF SCI_START,SCI_ENTRY,OUTPUT_A,OUTCRLF,OUTSTRG,ONSCI,SEND_A


;**********************
; Program starts here *
;**********************


;

; Initialize the SCI0 for 9600 baud
; Since we're using a 16.0 MHz clock and the Baud rate register is calculated:
; 	BR = MCLK / (16 * Baud_Rate)
;	MCLK = crystal / 2
;	BR = 8,000,000 / (16 * 9600) which is 153,600
;	BR = 52 which is 34 hex



myCode: section



SEND_A:
	pshd
	tab		; copy A into B
	LSRA
	LSRA
	LSRA
	LSRA		; allign most significant number (MSN)
	anda	#$0f
	andb	#$0f	; mask off unwanted numbers
	cmpa	#$0a	; if (MSN >= 9)
	bhs	ALPHA	;  convert to alpha
	adda	#$30	; else convert to number
	bra	NEXT
ALPHA:	adda	#$57
NEXT:	cmpb	#$0a	; if (LSN >= 9)
	bhs	ALPHA2	;  convert to alpha
	addb	#$30	; else convert to number
	bra	NEXT2
ALPHA2:	addb	#$57
NEXT2:	pshb
	jsr	OUTPUT_A
	pulb
	tba		; get the least significant number (LSN)
	jsr	OUTPUT_A
	puld		
	RTS

ONSCI:
	ldaa	#$34	; get baud rate constant
	staa	SC0BDL	; store low byte
	clr	SC0BDH	; clear high byte
	ldaa	#$00	; configure SCI0 control registers
	staa	SC0CR1
	ldaa	#$0C	; enable transmit and receive
	staa	SC0CR2
	RTS

; Output string of ASCII bytes starting at x until end of text ($04).
OUTSTRG:
	JSR  OUTCRLF	; output carriage-return
OUTSTRG0:
	PSHD          ; save a
OUTSTRG1:
	LDAA 0,X	; read char into a
        CMPA #EOT      ; is this end of text?
        BEQ  OUTSTRG3	; jump if yes
        JSR  OUTPUT_A	; output character
        INX            ; increment pointer
        BRA  OUTSTRG1  ; loop
OUTSTRG3:
	PULD		; restore a
        RTS

; Output a Carriage return and a line feed.  Returns a = cr.
OUTCRLF:
	PSHD
        LDAA #$0A       ; get LF
	JSR  OUTPUT_A	; send it
	LDAA #$0D	; get CR
	JSR  OUTPUT_A	; send it
        LDAA #$00
	JSR  OUTPUT_A	; output padding
	LDAA #$0D
	PULD
	RTS

; Output A to SCI0
OUTPUT_A:
OUTSCI2:
	LDAB	SC0SR1	; read status
	BITB	#$80	; test Transmit Data Register Empty bit
	BEQ	OUTSCI2	; loop if TDRE=1
	ANDA	#$7F	; mask parity
	STAA	SC0DRL	; send character
        RTS


SCI_ENTRY:
	LDS	#STACK	; initialize the stack pointer (don't do this if under monitor)
SCI_START:

	ldab	#$23
	JSR	ONSCI	; initialize serial port
	LDX	#LOCALMSG	; get message string
	JSR	OUTSTRG	; send it out serial port
	JSR	OUTCRLF	; output carriage-return
ENDPROG:
        RTS		; return (use this only if called, from monitor for example)


;**********************
; End of main loop    *
;**********************


;**************
; TEXT TABLES *
;**************

locData: section
 LOCALMSG:     dc.b   'Hello World this is a test'
         dc.b   EOT







    

